import { __decorate, __metadata } from 'tslib';
import { Prop, Watch, Component } from 'vue-property-decorator';
import { precisionRound, calculatePrecision } from '@uifabric-vue/utilities';
import BaseComponent from '../BaseComponent.js';
import { Icon } from '../Icon/index.js';
import { Position } from '../../utilities/positioning/positioning.types.js';
import { IconButton } from '../Button/IconButton/IconButton.js';
import { Label } from '../Label/index.js';
import { getClassNames } from './SpinButton.classNames.js';
import { getArrowButtonStyles, getStyles } from './SpinButton.styles.js';

var KeyboardSpinDirection;

(function (KeyboardSpinDirection) {
  KeyboardSpinDirection[KeyboardSpinDirection["down"] = -1] = "down";
  KeyboardSpinDirection[KeyboardSpinDirection["notSpinning"] = 0] = "notSpinning";
  KeyboardSpinDirection[KeyboardSpinDirection["up"] = 1] = "up";
})(KeyboardSpinDirection || (KeyboardSpinDirection = {}));

let SpinButton = class SpinButton extends BaseComponent {
  constructor() {
    super(...arguments);
    this.Position = Position;
    this.getArrowButtonStyles = getArrowButtonStyles;
    this.customUpArrowButtonStyles = {};
    this.customDownArrowButtonStyles = {};
    this.internalValue = this.value || this.defaultValue;
    this.INITIAL_STEP_DELAY = 400;
    this.STEP_DELAY = 75;
    this.isFocused = false;
    this.keyboardSpinDirection = KeyboardSpinDirection.notSpinning;
  }

  get classNames() {
    const {
      theme,
      styles,
      className,
      disabled,
      isFocused,
      keyboardSpinDirection,
      labelPosition
    } = this;
    return getClassNames(getStyles(theme, styles), disabled, isFocused, keyboardSpinDirection, labelPosition, className);
  }

  onValueChanged(value) {
    this.$emit('input', value);
  }

  updateValue(value) {
    let newValue = precisionRound(value, this.precision);
    this.internalValue = newValue;
    return newValue;
  }

  onIncrement() {
    let newValue = Math.min(this.internalValue + this.step, this.max);
    return this.updateValue(newValue);
  }

  onDecrement() {
    let newValue = Math.max(this.internalValue - this.step, this.min);
    return this.updateValue(newValue);
  }

  startSpin(dir) {
    let newValue = dir === 1 ? this.onIncrement() : this.onDecrement();
    newValue = precisionRound(newValue, this.precision);
    this.internalValue = newValue;
    this.timeout = setTimeout(() => {
      this.interval = setInterval(() => {
        let newValue = dir === 1 ? this.onIncrement() : this.onDecrement();
        newValue = precisionRound(newValue, this.precision);
        this.internalValue = newValue;
      }, this.STEP_DELAY);
    }, this.INITIAL_STEP_DELAY);
  }

  get precision() {
    const precision = Math.max(calculatePrecision(this.step), 0);
    return precision;
  }

  stopSpin() {
    clearTimeout(this.timeout);
    clearInterval(this.interval);
  }

  render() {
    const h = arguments[0];
    const {
      theme,
      customUpArrowButtonStyles,
      customDownArrowButtonStyles,
      classNames,
      labelPosition,
      iconProps,
      label,
      internalValue
    } = this;
    return h("div", {
      "class": classNames.root
    }, [labelPosition !== Position.bottom && (iconProps || label) && h("div", {
      "class": classNames.labelWrapper
    }, [iconProps && h(Icon, {
      "attrs": {
        "class-name": classNames.icon
      },
      "props": { ...iconProps
      }
    }), h(Label, {
      "attrs": {
        "for": `SpinButton${this.uid}`
      },
      "class": classNames.label
    }, [label])]), h("div", {
      "class": classNames.spinButtonWrapper
    }, [h("input", {
      "attrs": {
        "id": `SpinButton${this.uid}`,
        "type": "text",
        "role": "spinbutton"
      },
      "class": classNames.input,
      "domProps": {
        "value": internalValue
      },
      "on": {
        "focus": () => this.isFocused = true,
        "blur": () => this.isFocused = false,
        "input": ev => this.internalValue = +(ev.target.value || 0)
      }
    }), h("span", {
      "class": classNames.arrowBox
    }, [h(IconButton, {
      "class": "ms-UpButton",
      "attrs": {
        "styles": getArrowButtonStyles(theme, true, customUpArrowButtonStyles),
        "icon-props": {
          iconName: 'ChevronUpSmall'
        }
      },
      "nativeOn": {
        "mousedown": () => this.startSpin(1),
        "mouseup": () => this.stopSpin()
      }
    }), h(IconButton, {
      "class": "ms-DownButton",
      "attrs": {
        "styles": getArrowButtonStyles(theme, false, customDownArrowButtonStyles),
        "icon-props": {
          iconName: 'ChevronDownSmall'
        }
      },
      "nativeOn": {
        "mousedown": () => this.startSpin(-1),
        "mouseup": () => this.stopSpin()
      }
    })])]), labelPosition === Position.bottom && (iconProps || label) && h("div", {
      "class": classNames.labelWrapper
    }, [iconProps && h(Icon, {
      "attrs": {
        "class-name": classNames.icon
      },
      "props": { ...iconProps
      }
    }), label && h(Label, {
      "attrs": {
        "for": `SpinButton${this.uid}`
      },
      "class": classNames.label
    }, [label])])]);
  }

};

__decorate([Prop({
  default: false
}), __metadata("design:type", Boolean)], SpinButton.prototype, "disabled", void 0);

__decorate([Prop({
  default: false
}), __metadata("design:type", Boolean)], SpinButton.prototype, "required", void 0);

__decorate([Prop({
  default: 0
}), __metadata("design:type", Number)], SpinButton.prototype, "defaultValue", void 0);

__decorate([Prop({
  default: null
}), __metadata("design:type", Number)], SpinButton.prototype, "value", void 0);

__decorate([Prop({
  default: null
}), __metadata("design:type", String)], SpinButton.prototype, "label", void 0);

__decorate([Prop({
  default: Position.start
}), __metadata("design:type", Number)], SpinButton.prototype, "labelPosition", void 0);

__decorate([Prop({
  default: 0
}), __metadata("design:type", Number)], SpinButton.prototype, "min", void 0);

__decorate([Prop({
  default: 10
}), __metadata("design:type", Number)], SpinButton.prototype, "max", void 0);

__decorate([Prop({
  default: 1
}), __metadata("design:type", Number)], SpinButton.prototype, "step", void 0);

__decorate([Prop({
  type: Object,
  default: () => {}
}), __metadata("design:type", Object)], SpinButton.prototype, "iconProps", void 0);

__decorate([Watch('internalValue'), __metadata("design:type", Function), __metadata("design:paramtypes", [String]), __metadata("design:returntype", void 0)], SpinButton.prototype, "onValueChanged", null);

SpinButton = __decorate([Component({
  components: {
    IconButton,
    Icon,
    Label
  }
})], SpinButton);

export { KeyboardSpinDirection, SpinButton };
